import { __awaiter } from "tslib";
import { DefaultApiFactory } from "@qorebase/sdk";
import { tableParams } from "../../utils/tableParams";
export const insightSourceFetcher = (source, schema, config) => {
    var _a;
    const sdk = DefaultApiFactory({
        apiKey: config.adminSecret,
        isJsonMime: (mime) => {
            const jsonMime = new RegExp("^(application/json|[^;/ \\t]+/[^;/ \\t]+[+]json)[ \\t]*(;.*)?$", "i");
            return (mime !== null &&
                (jsonMime.test(mime) ||
                    mime.toLowerCase() === "application/json-patch+json"));
        },
        accessToken: (_a = config.getToken) === null || _a === void 0 ? void 0 : _a.call(config),
    }, config.endpoint);
    const params = tableParams(schema).map((param) => ({
        slug: param.name,
        type: param.type,
        required: true,
    })) || [];
    const inputParams = schema.columns.map((field) => ({
        slug: field.name,
        type: field.type,
        required: true,
    })) || [];
    const fields = schema.columns.map((field) => ({
        id: field.name,
        display: field.name,
        type: field.type,
    }));
    const fetch = (params) => __awaiter(void 0, void 0, void 0, function* () {
        if (!(source === null || source === void 0 ? void 0 : source.target))
            return {};
        const result = yield sdk.execute({
            operations: [
                {
                    operation: "Insight",
                    instruction: {
                        name: "data",
                        insight: source.target,
                        start: params === null || params === void 0 ? void 0 : params.start,
                        end: params === null || params === void 0 ? void 0 : params.end,
                        params: source.params,
                    },
                },
            ],
        });
        return result.data.results.data[0];
    });
    const fetchList = (params) => __awaiter(void 0, void 0, void 0, function* () {
        if (!(source === null || source === void 0 ? void 0 : source.target))
            return [];
        const result = yield sdk.execute({
            operations: [
                {
                    operation: "Insight",
                    instruction: {
                        name: "data",
                        insight: source.target,
                        start: params === null || params === void 0 ? void 0 : params.start,
                        end: params === null || params === void 0 ? void 0 : params.end,
                        params: source.params,
                    },
                },
            ],
        });
        return result.data.results.data;
    });
    const store = (params) => __awaiter(void 0, void 0, void 0, function* () {
        throw Error("Not implemented");
    });
    const del = (params) => __awaiter(void 0, void 0, void 0, function* () {
        throw Error("Not implemented");
    });
    return {
        type: "insight",
        params,
        inputParams,
        fields,
        fetch,
        fetchList,
        store,
        del,
    };
};
//# sourceMappingURL=insightSourceHooks.js.map