import { __awaiter, __rest } from "tslib";
import { DefaultApiFactory } from "@qorebase/sdk";
import { tableParams } from "../../utils/tableParams";
export const transformRow = (endpoint, storageToken, schema, row) => {
    const columnMap = Object.fromEntries(schema.columns.map((column) => [column.name, column]) || []);
    return Object.fromEntries(Object.entries(row).map(([key, val]) => {
        var _a, _b, _c, _d;
        if (!(val && typeof val === "object" && "filename" in val))
            return [key, val];
        if (storageToken && "url" in val) {
            return [key, `${val.url}?token=${storageToken}`];
        }
        const column = columnMap[key];
        const tableName_ = (column === null || column === void 0 ? void 0 : column.type) === "lookup" &&
            ((_b = columnMap[(_a = column.definition) === null || _a === void 0 ? void 0 : _a.relation]) === null || _b === void 0 ? void 0 : _b.references)
            ? columnMap[(_c = column.definition) === null || _c === void 0 ? void 0 : _c.relation].references.targetTable
            : "query" in schema && typeof schema.query !== "string"
                ? schema.query.table
                : schema.name;
        const rowId = ((column === null || column === void 0 ? void 0 : column.type) === "lookup" &&
            row[(_d = column.definition) === null || _d === void 0 ? void 0 : _d.relation]) ||
            row.id;
        const columnName = ((column === null || column === void 0 ? void 0 : column.type) === "lookup" && column.definition.column) || key;
        return [
            key,
            `${endpoint}/v1/files/public/${tableName_}/${rowId}/${columnName}/${
            // @ts-ignore
            val.filename}`,
        ];
    }));
};
export const tableSourceFetcher = (source, storageToken, schema, config) => {
    var _a;
    const rowId = "rowId" in source ? source.rowId : null;
    const sdk = DefaultApiFactory({
        apiKey: config.adminSecret,
        isJsonMime: (mime) => {
            const jsonMime = new RegExp("^(application/json|[^;/ \\t]+/[^;/ \\t]+[+]json)[ \\t]*(;.*)?$", "i");
            return (mime !== null &&
                (jsonMime.test(mime) ||
                    mime.toLowerCase() === "application/json-patch+json"));
        },
        accessToken: (_a = config.getToken) === null || _a === void 0 ? void 0 : _a.call(config),
    }, config.endpoint);
    const transformRow_ = (row) => transformRow(config.endpoint, storageToken, schema, row);
    const params = tableParams(schema).map((param) => ({
        slug: param.name,
        type: param.type,
        required: true,
    })) || [];
    const inputParams = schema.columns.map((field) => ({
        slug: field.name,
        type: field.type,
        required: true,
    })) || [];
    const fields = schema.columns.map((field) => ({
        id: field.name,
        display: field.name,
        type: field.type,
    }));
    const fetch = (params) => __awaiter(void 0, void 0, void 0, function* () {
        const defaultRow = (source.type === "record" &&
            source.defaultRow.type === "row-id" &&
            source.defaultRow.rowId) ||
            null;
        const fetchFirstRow = source.type === "record" && source.defaultRow.type === "first-row";
        if (!(source === null || source === void 0 ? void 0 : source.target))
            return {};
        const result = yield sdk.execute({
            operations: [
                {
                    operation: "Select",
                    instruction: {
                        table: source.target,
                        name: "data",
                        condition: fetchFirstRow
                            ? undefined
                            : {
                                $and: [
                                    {
                                        id: {
                                            $eq: rowId || defaultRow,
                                        },
                                    },
                                ],
                            },
                        limit: 1,
                    },
                },
            ],
        });
        return transformRow_(result.data.results.data[0]);
    });
    const fetchList = (params) => __awaiter(void 0, void 0, void 0, function* () {
        var _b;
        const sourceList = source.type === "list" || source.type === "standalone-list"
            ? source
            : source.type === "record-list" && source.sourceType === "list"
                ? Object.assign(Object.assign({}, source), { type: "list" })
                : null;
        if (!(source === null || source === void 0 ? void 0 : source.target))
            return [];
        const sourceListOpts = sourceList.type === "list"
            ? {
                limit: sourceList === null || sourceList === void 0 ? void 0 : sourceList.limit,
                offset: sourceList === null || sourceList === void 0 ? void 0 : sourceList.offset,
            }
            : {};
        const result = yield sdk.execute({
            operations: [
                {
                    operation: "Select",
                    instruction: {
                        table: source.target,
                        name: "data",
                        params: (sourceList === null || sourceList === void 0 ? void 0 : sourceList.params) || {},
                        orderBy: ((_b = sourceList === null || sourceList === void 0 ? void 0 : sourceList.sorts) === null || _b === void 0 ? void 0 : _b.field)
                            ? {
                                [sourceList.sorts.field]: sourceList.sorts.type,
                            }
                            : {},
                        limit: sourceListOpts.limit,
                        offset: sourceListOpts.offset,
                    },
                },
            ],
        });
        return result.data.results.data.map(transformRow_);
    });
    const store = (params) => __awaiter(void 0, void 0, void 0, function* () {
        const { id } = params, params_ = __rest(params, ["id"]);
        // TODO: store for view should insert to view instead of table
        const result = yield sdk.execute({
            operations: [
                id
                    ? {
                        operation: "Update",
                        instruction: {
                            table: source.target,
                            name: "data",
                            condition: {
                                id,
                            },
                            set: params_,
                        },
                    }
                    : {
                        operation: "Insert",
                        instruction: {
                            table: source.target,
                            name: "data",
                            data: params_,
                        },
                    },
            ],
        });
        return result.data.results.data[0];
    });
    const del = (params) => __awaiter(void 0, void 0, void 0, function* () {
        const { id } = params;
        yield sdk.execute({
            operations: [
                {
                    operation: "Delete",
                    instruction: {
                        table: source.target,
                        name: "data",
                        condition: { id },
                    },
                },
            ],
        });
    });
    return {
        type: "table",
        params,
        inputParams,
        fields,
        fetch,
        fetchList,
        store,
        del,
    };
};
//# sourceMappingURL=tableSourceHooks.js.map