import { __awaiter } from "tslib";
// @ts-nocheck
import React from "react";
import { componentAliases } from "../componentAliases";
import { registerComponent, getComponent, } from "../interfaces";
import { RuntimeContext } from "../RuntimeContext";
import { importCustomComponent, importNpmComponent } from "./importComponent";
import { mutate } from "swr";
import useSWRImmutable from "swr/immutable";
import { builtinComponents } from "../registeredComponents/builtin";
export const useImportComponentLazy = () => {
    const npmComponents = RuntimeContext.useSelectState((state) => state.components);
    return React.useCallback((name) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const registeredComponent = getComponent(name);
        if (registeredComponent) {
            return registeredComponent;
        }
        const npmComponent = npmComponents.find((component) => component.name === name && component.enabled);
        if (npmComponent) {
            return (yield importNpmComponent(name, npmComponent.version)).component;
        }
        const builtinComponent = (_a = builtinComponents[name]) !== null && _a !== void 0 ? _a : builtinComponents[componentAliases[name]];
        if (!builtinComponent) {
            throw Error(`Component ${name} not found`);
        }
        return builtinComponent.importComponent().then((module) => {
            mutate(["npm components", name], module.default);
            registerComponent(name, module.default).component;
        });
    }), [npmComponents]);
};
export const useImportComponent = (instance) => {
    const importComponent = useImportComponentLazy();
    const { data } = useSWRImmutable(["npm components", instance], (_, instance) => instance.metadata.componentURL
        ? importCustomComponent(instance.metadata.componentURL).then((component) => component.component)
        : importComponent(instance.componentID), {
        isPaused: () => !(instance === null || instance === void 0 ? void 0 : instance.componentID),
        // enable suspense if not custom component
        suspense: !instance.metadata.componentURL,
    });
    return data;
};
export const importBuiltins = (ignores) => __awaiter(void 0, void 0, void 0, function* () {
    return Promise.all(Object.entries(builtinComponents)
        .filter(([name]) => !ignores.includes(name))
        .map(([name, { importComponent }]) => importComponent().then((module) => {
        mutate(["npm components", name], module.default);
        return registerComponent(name, module.default);
    })));
});
//# sourceMappingURL=builtinComponents.js.map